unit HTMLTemplate;

interface

procedure OpenHTMLTemplate(Template,Output:string;Pics,C,L:integer);
procedure SetHTMLToken(Token,Value:string);
procedure StartTemplateLoop;
procedure NextTemplateLoop;
procedure CloseHTMLTemplate;
function IIf(Cond:Boolean;TrueP,FalseP:Variant):Variant;

implementation

uses Classes,SysUtils,Dialogs,Math;

const
  TP='__';
  LO='LOOP';
  BR='BREAK';
  AL='ALLLINKS';
  PA='PAGE';
  MA='MAXPAGE';

var
  BeforeLoop, Loop, Break, AfterLoop, OutFile,
  AktLoop, AktContent:string;
  Pages,Cols,Lines, AktFileNum, AktCol, AktLine:integer;
  Looping:boolean;

function ReadFullFile(FileName:string):string;
var
  FileStream:TFileStream;
  Buffer:PChar;
begin
  FileStream:=TFileStream.Create(FileName,fmOpenRead or fmShareDenyWrite);
  try
    Buffer:=PChar(AllocMem(FileStream.Size+1));
    Result:=IntToStr(FileStream.Read(Buffer^,FileStream.Size));
    Result:=string(Buffer);
  finally
    FileStream.Free;
  end;
end;

procedure WriteFullFile(FileName,Content:string);
var
  FileStream:TFileStream;
begin
  FileStream:=TFileStream.Create(FileName,fmCreate or fmShareExclusive);
  try
    FileStream.Write(PChar(Content)^,Length(Content));
  finally
    FileStream.Free;
  end;
end;

function TokenNotFound(Token,SearchText:string; Complain:Boolean):Boolean;
begin
  Result:=false;
  if Pos(TP+Token+TP,SearchText)=0 then
    begin
    if Complain then
      MessageDlg('Der Token '+TP+Token+TP+' wurde in der Vorlage nicht '+
        'oder an falscher Stelle gefunden!',mtError,[mbOK],0);
    Result:=true;
    end;
end;

function SearchReplace(InStr,Search,Replace:string):string;
begin
  Result:='';
  while Pos(Search,InStr)<>0 do
    begin
    Result:=Result+Copy(InStr,1,Pos(Search,InStr)-1)+Replace;
    Delete(InStr,1,Pos(Search,InStr)+Length(Search)-1);
    end;
  Result:=Result+InStr;
end;

procedure GoToNewFile;
var
  Prefix,Suffix,Links:string;
  i:integer;
begin
  Prefix:=Copy(OutFile,0,Length(OutFile)-Length(ExtractFileExt(OutFile)));
  Suffix:=ExtractFileExt(OutFile);

  Links:='';
  for i:=1 to Pages do
    if i=AktFileNum then
      Links:=Links+'<b>'+IntToStr(i)+'</b> '
    else
      Links:=Links+'<a href="'+ExtractFileName(Prefix)+
        IIf(i=1,'',IntToStr(i))+Suffix+'">'+IntToStr(i)+'</a> ';

  AktContent:=AktContent+SearchReplace(AfterLoop,TP+PA+TP,IntToStr(AktFileNum));
  AktContent:=SearchReplace(AktContent,TP+AL+TP,Links);

  if AktFileNum<>0 then WriteFullFile(Prefix+
    IIf(AktFileNum=1,'',IntToStr(AktFileNum))+Suffix,AktContent);

  Inc(AktFileNum);
  AktContent:=SearchReplace(BeforeLoop,TP+PA+TP,IntToStr(AktFileNum));
end;

procedure OpenHTMLTemplate(Template,Output:string;Pics,C,L:integer);
var
  TextA:string;
begin

  // Read file and cut it in pieces
  TextA:=ReadFullFile(Template);

  if TokenNotFound(LO,TextA,true) then Exit;
  BeforeLoop:=Copy(TextA,1,Pos(TP+LO+TP,TextA)-1);
  Delete(TextA,1,Pos(TP+LO+TP,TextA)+Length(TP+LO+TP)-1);

  if TokenNotFound(LO,TextA,true) then Exit;

  if TokenNotFound(BR,TextA,false) then
    begin
    Loop:=Copy(TextA,1,Pos(TP+LO+TP,TextA)-1);
    Delete(TextA,1,Pos(TP+LO+TP,TextA)+Length(TP+LO+TP)-1);
    Break:='';
    end
  else
    begin
    Loop:=Copy(TextA,1,Pos(TP+BR+TP,TextA)-1);
    Delete(TextA,1,Pos(TP+BR+TP,TextA)+Length(TP+BR+TP)-1);
    Break:=Copy(TextA,1,Pos(TP+LO+TP,TextA)-1);
    Delete(TextA,1,Pos(TP+LO+TP,TextA)+Length(TP+LO+TP)-1);
    end;

  AfterLoop:=TextA;

  // Set other variables
  Pages:=Ceil(Pics/(C*L));
  Cols:=C;
  Lines:=L;
  OutFile:=Output;
  AktFileNum:=0;
  AktCol:=1;
  AktLine:=1;

  Looping:=false;

  SetHTMLToken(MA,IntToStr(Pages));
end;

procedure SetHTMLToken(Token,Value:string);
begin
  if Looping then
    AktLoop:=SearchReplace(AktLoop,TP+Token+TP,Value)
  else
    begin
    BeforeLoop:=SearchReplace(BeforeLoop,TP+Token+TP,Value);
    AfterLoop:=SearchReplace(AfterLoop,TP+Token+TP,Value);
    end;
end;

procedure StartTemplateLoop;
begin
  Looping:=true;
  AktLoop:=Loop;
  GoToNewFile;
end;

procedure NextTemplateLoop;
begin
  AktContent:=AktContent+AktLoop;
  AktLoop:=Loop;

  Inc(AktCol);
  if AktCol>Cols then
    begin
    Inc(AktLine);
    AktCol:=1;
    AktContent:=AktContent+Break;
    end;

  if AktLine>Lines then
    begin
    AktLine:=1;
    GoToNewFile;
    end;
end;

procedure CloseHTMLTemplate;
begin
  GoToNewFile;
end;

function IIf(Cond:Boolean;TrueP,FalseP:Variant):Variant;
begin
  if Cond then
    Result:=TrueP
  else
    Result:=FalseP;
end;

end.
