unit ImageFuncs;

interface

uses Windows,Graphics;

procedure InitImage;
procedure FreeImage;
procedure LoadImage(FileName:string);
procedure SaveImage(FileName:string);
procedure CropImage(Left,Right,Top,Bottom:integer);
procedure ZoomImage(H,V:single);
procedure FitToSizeImage(Width,Height:integer);
procedure GetImageSize(var Width,Height:integer);
function FindNewFileName(Name:string):string;
function FullDir(Dir:string):string;
function GetFileSize (const FName: string): integer;

var MainImage : TBitmap;

implementation

uses SysUtils,JPEG,GifImage,Dialogs;

// Initalize the image
procedure InitImage;
begin
  MainImage:=TBitmap.Create;
end;

// Remove the image from memory
procedure FreeImage;
begin
  MainImage.Free;
end;

// Load an image file
procedure LoadImage(FileName:string);
var
  ext : string;
  bmp : TBitmap;
  gif : TGifImage;
  jpg : TJPEGImage;
begin
  ext:=LowerCase(ExtractFileExt(FileName));

  if ext='.bmp' then
    begin
    bmp:=TBitmap.Create;
    bmp.LoadFromFile(FileName);
    MainImage.Assign(bmp);
    bmp.Free;
    end {.bmp}
  else if ext='.gif' then
    begin
    gif:=TGifImage.Create;
    gif.LoadFromFile(FileName);
    gif.OptimizeColorMap;
    MainImage.Assign(gif);
    gif.Free;
    end {.gif}
  else if (ext='.jpg') or (ext='.jpeg') then
    begin
    jpg:=TJPEGImage.Create;
    jpg.LoadFromFile(FileName);
    MainImage.Assign(jpg);
    jpg.Free;
    end {.jpg}
  else
    begin
    MessageDlg('Dateityp nicht untersttzt!!! Bitte in der Dokumentation nachsehen!',mtError,[mbYes,mbOK],0);
    Exit;
    end;
end;

// Save an image file
procedure SaveImage(FileName:string);
var
  ext : string;
  bmp : TBitmap;
  gif : TGifImage;
  jpg : TJPEGImage;
begin
  ext:=LowerCase(ExtractFileExt(FileName));

  if ext='.bmp' then
    begin
    bmp:=TBitmap.Create;
    bmp.Assign(MainImage);
    bmp.SaveToFile(FileName);
    bmp.Free;
    end {.bmp}
  else if ext='.gif' then
    begin
    gif:=TGifImage.Create;
    gif.ColorReduction:=rmNone;
    gif.Assign(MainImage);
    gif.SaveToFile(FileName);
    gif.Free;
    end {.gif}
  else if (ext='.jpg') or (ext='.jpeg') then
    begin
    jpg:=TJPEGImage.Create;
    jpg.Assign(MainImage);
    jpg.Compress;
    jpg.SaveToFile(FileName);
    jpg.Free;
    end {.jpg}
  else
    begin
    MessageDlg('Dateityp nicht untersttzt!!! Bitte in der Dokumentation nachsehen!',mtError,[mbYes,mbOK],0);
    Exit;
    end;
end;

// Crop image by some pixels
procedure CropImage(Left,Right,Top,Bottom:integer);
begin
  MainImage.Canvas.Draw(-Left,-Top,MainImage);
  MainImage.Width:=MainImage.Width-Left-Right;
  MainImage.Height:=MainImage.Height-Top-Bottom;
end;

// Resize image by H x V percent
procedure ZoomImage(H,V:single);
var
  R : TRect;
begin
  R.Left:=0;
  R.Top:=0;
  // Make bigger?
  if V > 100 then
    MainImage.Width:=Round(MainImage.Width*V/100);
  if H > 100 then
    MainImage.Height:=Round(MainImage.Height*H/100);
  // Calculate new size...
  R.Right:=Round(MainImage.Width*V/100);
  R.Bottom:=Round(MainImage.Height*H/100);
  // Draw
  MainImage.Canvas.StretchDraw(R,MainImage);
  // Make smaller?
  if V < 100 then
    MainImage.Width:=R.Right;
  if H < 100 then
    MainImage.Height:=R.Bottom;
end;

// Resize image by H x V percent
procedure FitToSizeImage(Width,Height:integer);
var
  R : TRect;
begin
  R.Left:=0;
  R.Top:=0;
  // Make bigger?
  if Width > MainImage.Width then
    MainImage.Width:=Width;
  if Height > MainImage.Height then
    MainImage.Height:=Height;
  // Set Rect
  R.Right:=Width;
  R.Bottom:=Height;
  // Draw
  MainImage.Canvas.StretchDraw(R,MainImage);
  // Make smaller?
  if Width < MainImage.Width then
    MainImage.Width:=Width;
  if Height < MainImage.Height then
    MainImage.Height:=Height;
end;

procedure GetImageSize(var Width,Height:integer);
begin
  Width:=MainImage.Width;
  Height:=MainImage.Height;
end;

// This function finds a new filename for a file, if the filename is already
// used, by adding a underscore and a number to the filename
function FindNewFileName(Name:string):string;
var
  T:string;
  num:integer;
begin
  T:=Name;
  num:=1;
  while FileExists(T) do
    begin
    T:=Copy(Name,0,Length(Name)-Length(ExtractFileExt(Name)))+
       '_'+IntToStr(num)+ExtractFileExt(Name);
    Inc(num);
    end;
  FindNewFileName:=T;
end;

// Append a backslash to a directory name, if there is no
function FullDir(Dir:string):string;
begin
  if Copy(Dir,Length(Dir),1)='\' then
    FullDir:=Dir
  else
    Fulldir:=Dir+'\';
end;

function GetFileSize (const FName: string): integer;
var
  SearchRec: TSearchRec;
begin
  if FindFirst (FName, faAnyFile, SearchRec) = 0 then begin
    Result := SearchRec.Size;
  end else Result := 0;
  FindClose (SearchRec);
end;

end.
